'use strict';
(function() {

//=================//
// Global Constant //
//=================//
// Count players
const playerClass = document.getElementsByClassName('shap');
const pcl = playerClass.length;

// Get elements
const audioTag   = document.getElementsByTagName('audio');
const playId     = document.getElementsByClassName('play');
const pauseId    = document.getElementsByClassName('pause');
const currentId  = document.getElementsByClassName('current');
const durationId = document.getElementsByClassName('duration');
const seekBarId  = document.getElementsByClassName('seekbar');
const volumeId   = document.getElementsByClassName('volumebar');

// Set icons
// These charactors are UTF-8 by ./font_button.woff2
const playIcon  = '▶';
const stopIcon  = '■';
const pauseIcon = '〓';

// Set fonts
// If in case use custum fonts
// Use function rotateIcon for '〓'
// Use function changeActiveColor then set text-indent for playId


//===========//
// Main Code //
//===========//
function main() {
  // Make <div class="shap><audio src="./hoge.mp3">... for html
  makePlayer();

  // Make operation for each player
  for (let i = 0; i < pcl; i++) {
    playButton(i);
    stopButton(i);
    setPlayerTime(i);
    operateSeekBar(i);
    operateVolumeBar(i);
    setInitialVolumeValue(i);
    checkDurationTime(i); // Check just in case (Surely get)
  }

  // Check just in case (Surely gets duration time)
  //for (let i = 0; i < pcl; i++) {
  //  checkDurationTime(i);
  //}
}


function makePlayer() {
  for (let i = 0; i < pcl; i++) {
    playerClass[i].setAttribute('id', 'audio' + i);
    let audioSrc = playerClass[i].getAttribute('data-src');

    playerClass[i].innerHTML
      = '<audio src="' + audioSrc + '" controls></audio>'
      + '<div class="ctrl">'
      + '<div class="play" id="play' + [i] + '">' + playIcon + '</div>'
      + '<div class="pause" id="pause' + [i] + '">' + stopIcon + '</div>'
      + '</div>'
      + '<div class="info">'
      + '<span class="current" id="current' + [i] + '">00:00</span>'
      + '<span class="duration" id="duration' + [i] + '">--:--</span>'
      + '<span class="volume"><span class="volume3"></span></span>'
      + '<span class="minus">-</span>'
      + '<span class="plus">+</span>'
      + '</div>'
      + '<input class="seekbar" type="range" min="0" max="100" value="0" id="seekbar' + [i] + '"></input>'
      + '<input class="volumebar" type="range" min="0" max="0.98" value="0.8" step="0.01" id="volume' + [i] + '"></input>';  // 0.98 guard over 1
  }
}


function playButton(i) {
  playId[i].addEventListener('click', function() {
    // Initial player. Stop other player is playing
    initPlayer(i);

    // "paused" return boolean. pause is true, play is false.
    if (audioTag[i].paused) {
      fade(i, 'in');
      audioTag[i].play();
      ifEnded(i);
      playId[i].innerHTML = playId[i].innerHTML === playIcon ? pauseIcon : playIcon;
      changeActiveColor(i, 'active');

    } else {
      fade(i, 'out');
      wait(i, 'pause');

      playId[i].innerHTML = playIcon;
      changeActiveColor(i, 'inactive');
    }
  });
}


function stopButton(i) {
  pauseId[i].addEventListener('click', function() {
    // Only click pause button of active player
    if (playId[i].innerHTML === playIcon) {
      audioTag[i].currentTime = 0;
      seekBarId[i].value = 0;

      return;
    }

    fade(i, 'out');
    wait(i, 'stop')

    playId[i].innerHTML = playIcon;  // init play synbol direction
    changeActiveColor(i, 'inactive');
  });
}


function ifEnded(i) {
  audioTag[i].addEventListener('ended', function() {
    if (playId[i].innerHTML === pauseIcon) {
      audioTag[i].pause();
      audioTag[i].currentTime = 0;
      changeActiveColor(i, 'inactive');
      playId[i].innerHTML = playId[i].innerHTML === playIcon ? pauseIcon : playIcon;
    }
  }, false);
}


function fade(i, s) {
  let   maxVol    = volumeId[i].value;
  const minVol    = 0.01; // Guard under 0
  const adjustVol = 0.01;
  const fadeSpeed = 4;
  //  fadeSpeed = 4 is waitTime = 400 from wait()

  switch (s) {
    case 'in':
      audioTag[i].volume = 0;
      let startFunc = setInterval(function() {
        audioTag[i].volume += adjustVol;

        if (audioTag[i].volume >= maxVol) {
          audioTag[i].volume = maxVol;
          clearInterval(startFunc);
        }
      }, fadeSpeed / maxVol);
      break;

    case 'out':
      let endFunc = setInterval(function() {
        try {
          audioTag[i].volume -= adjustVol;
        }
        catch (e) {
          audioTag[i].volume = 0;
        }

        if (minVol >= audioTag[i].volume) {
          audioTag[i].volume = 0;
          clearInterval(endFunc);
        }
      }, fadeSpeed / maxVol);
      break;
  }
}


function initPlayer(i) {
  if (playId[i].innerHTML === pauseIcon) {
    return;
  }

  for (let j = 0; j < pcl; j++) {
    if (playId[j].innerHTML === pauseIcon) {
      playId[j].click();
    }
  }
}


function setPlayerTime(i) {
  audioTag[i].addEventListener('loadedmetadata', function() {
    setDurationTime(i);
  });

  audioTag[i].addEventListener('timeupdate', (e) => {
    setCurrentTime(i);
    setSeekBarTime(i);
  });
}


function setDurationTime(i) {
  const duration = audioTag[i].duration;
  durationId[i].innerHTML = convertTime(duration);
  console.log(typeof audioTag[i].duration);
  console.log(typeof duration);
  console.log(typeof durationId[i].innerHTML);

  // Set max & step
  seekBarId[i].setAttribute('max', duration);
  seekBarId[i].setAttribute('step', 'any');
}


function setCurrentTime(i) {
  const current = Math.ceil(seekBarId[i].value);

  currentId[i].innerHTML = convertTime(current);
}


function setSeekBarTime(i) {
  seekBarId[i].value = audioTag[i].currentTime;
}


function operateSeekBar(i) {
  // Operation for PC & smartphone with click & touch
  seekBarId[i].addEventListener('pointerdown', (e) => {
    fade(i, 'out');
    wait(i, 'pause');
  });

  // Measures when mash seek bar
  seekBarId[i].addEventListener('pointerup', (e) => {
    if (playId[i].innerHTML === pauseIcon) {
      audioTag[i].volume = 0;
      fade(i, 'in');
      wait(i, 'play');
    }
  });

  // Operation for input type="range"
  seekBarId[i].addEventListener('input', (e) => {
    // Not change playback rate when move seek
    audioTag[i].volume = 0;

    if (playId[i].innerHTML === pauseIcon) {

      setCurrentTime(i);
      audioTag[i].currentTime = seekBarId[i].value;  // Substance of time
    } else if (playId[i].innerHTML === playIcon) {
      setCurrentTime(i);
      console.log(seekBarId[i].value);
    }
  });

  seekBarId[i].addEventListener('change', (e) => {
    if (playId[i].innerHTML === pauseIcon) {
      audioTag.volume = 0;
      audioTag[i].currentTime = seekBarId[i].value;
      fade(i, 'in');
      wait(i, 'play');
    } else if (playId[i].innerHTML === playIcon) {
      audioTag[i].currentTime = seekBarId[i].value;
    }
  });
}


function operateVolumeBar(i) {
  volumeId[i].addEventListener('input', (e) => {
    audioTag[i].volume = volumeId[i].value;
  });
}


function setInitialVolumeValue(i) {
  audioTag[i].volume = 0.8;
}


function checkDurationTime(i) {
  const duration = convertTime(Math.ceil(audioTag[i].duration));

  if (!Number.isNaN(audioTag[i].duration)) {
    durationId[i].innerHTML = duration;
  }
}


function wait(i, s) {
  const waitTime = 400;

  setTimeout(function() {
    switch (s) {
      case 'play':
        audioTag[i].play();
        break;

      case 'pause':
        audioTag[i].pause();
        break;

      case 'stop':
        audioTag[i].pause();
        audioTag[i].currentTime = 0;
        break;
    }
  }, waitTime);
}


function convertTime(t) {
  t = Math.ceil(t);
  let ms = '';

  const m = t / 60 | 0;
  const s = t % 60;
  const z2 = (v) => {
    const s = '00' + v;

    return s.substr(s.length - 2, 2);
  }

  if (m) {
    ms = z2(m) + ':' + z2(s);
  } else {
    ms = '00:' + z2(s);
  }

  return ms;
}


function changeActiveColor(i, s) {
  // Color
  const active = 'dodgerblue';
  const inactive = 'darkgray';

  switch (s) {
    case 'active':
      playId[i].setAttribute("style", "background:" + active);
      // + active + "; text-indent: 0;");  // When using a custum font
      break;

    case 'inactive':
      playId[i].setAttribute("style", "background:" + inactive);
      // + active + "; text-indent: .2em;");  // When using a custum font
      break;
  }
}


main();
}());
